<?php

/**
 * @file
 * Implements number of related form building functions and validate / submit hanlders.
 */

/**
 * Base route control form.
 *
 * @param Array $form
 * @param Array $form_state
 * @param Array $sub_form_id
 * @return Array
 */
function route_control_form(&$form, &$form_state) {
  $route = $form_state['route'];
  $info = $form_state['form_info'];
  ctools_include('form');
  $form['buttons']['#weight'] = 999;

  // Show only, if there's actually previous step available
  if ($route->get_previous_step()) {
    $form['buttons']['back'] = array(
      '#type' => 'submit',
      '#value' => t('Back'),
      '#validate' => array(),
      '#attributes' => array('class' => array('route-back')),
      '#submit' => array('route_control_form_back_submit', 'route_control_form_set_cache'),
    );
  }

  // Show only, if there's actually next step available
  if ($route->get_next_step()) {
    $form['buttons']['continue'] = array(
      '#type' => 'submit',
      '#value' => t('Continue'),
      '#validate' => array(),
      '#attributes' => array('class' => array('route-continue')),
      '#submit' => array('route_control_form_continue_submit', 'route_control_form_set_cache'),
    );
  }

  // Show only, if there's no next step available. This means we're finishing the journey
  else {
    $form['buttons']['continue'] = array(
      '#type' => 'submit',
      '#value' => t('Finish'),
      '#validate' => array(),
      '#submit' => array('route_control_form_finish_submit', 'route_control_form_set_cache'),
    );
  }

  // Wire up any validate and submit handlers.
  // In order for this to work, developers will have to stick to naming convention
  if (isset($form['buttons']['continue'])) {
    if (function_exists($info['form_id'] . '_validate')) {
      array_unshift($form['buttons']['continue']['#validate'], $info['form_id'] . '_validate');
    }
    if (function_exists($info['form_id'] . '_submit')) {
      array_unshift($form['buttons']['continue']['#submit'], $info['form_id'] . '_submit');
    }
  }

  return $form;
}

/**
 * Base route control form submit handler
 * @param Array $form
 * @param Array $form_state
 */
function route_control_form_back_submit($form, &$form_state) {
  $route = $form_state['route'];

  // Call back hook
  module_invoke_all('route_back', $route);

  // Regress to the previous step.
  if (!$form_state['redirect'] && $previous_step = $route->get_previous_step()) {
    // If we dont deprocess step here we can still jump to any part of the journey in the breadcrumb.
    // Check if this is what we want?
    $route->deprocess_step();
    $form_state['redirect'] = route_build_path(route_get_current_page_subtask_name(), $previous_step);
  }
}

/**
 * Base route control form submit handler
 * @param Array $form
 * @param Array $form_state
 */
function route_control_form_continue_submit($form, &$form_state) {
  $route = $form_state['route'];

  // Store URL query string as we are going to remove destination param to avoid redirect.
  //$query_string = drupal_get_query_parameters();

  // Unset destination to avoid any redirect in middle way.
  unset($_GET['destination']);


  // Progress to the next step and process
  if (!$form_state['redirect'] && !$form_state['rebuild'] && $next_step_url = _route_prepare_for_next_step($route, $form_state['values']['next_step'])) {
    $form_state['redirect'] = $next_step_url; //array($next_step_url, array('query' => $query_string));
  }
}


/**
 * Helper function to prepare the multistep panels controller to move to the next step in the path. Can be used
 * to progress multistep panels onto the next step when you don't have a multistep panels control form
 *
 * @param $context - The context object
 * @param $next_step - The name of the next step to progress to. Default is NULL, which means get from the $context
 *
 * @return string - The url of the next step
 */
function _route_prepare_for_next_step($context, $next_step = NULL) {

  if (!$next_step) {
    $next_step = $context->get_next_step();
  }

  // Call next hook
  module_invoke_all('route_next', $context);

  $context->process_step($next_step);
  //multistep_panels_set_context_cache(multistep_panels_get_token(), $context);
  return route_build_path(route_get_current_page_subtask_name(), $next_step);

}

/**
 * Base route control form finish handler
 * @param Array $form
 * @param Array $form_state
 */
function route_control_form_finish_submit($form, &$form_state) {
  $route = $form_state['route'];

  // Call finish hook
  module_invoke_all('route_finish', $route);

  // We are at the end, reset the route and go back to the start if the user clicks finish.
  if (!$form_state['redirect'] && !$form_state['rebuild']) {
    $route->finish();
    $route->reset();
    $form_state['redirect'] = route_build_path(route_get_current_page_subtask_name(), $route->get_first_step());
  }
}

/**
 * Submit handler that saves route object
 * @param Array $form
 * @param Array $form_state
 */
function route_control_form_set_cache($form, &$form_state) {
  route_set(route_token(), $form_state['route']);
}
